@@ -1,4 +1,4 @@
-from __future__ import division
+from __future__ import division, print_function
 
 import re
 import glob
@@ -10,6 +10,8 @@ from unittest import TestCase as BaseTestCase
 suites = []
 add = suites.append
 
+from mutagen._compat import cmp
+
 
 class TestCase(BaseTestCase):
 
@@ -17,7 +19,7 @@ class TestCase(BaseTestCase):
         def wrapped(*args, **kwargs):
             try:
                 fun(*args, **kwargs)
-            except Exception, e:
+            except Exception as e:
                 self.failUnless(re.search(re_, str(e)))
                 raise
         self.failUnlessRaises(exc, wrapped, *args, **kwargs)
@@ -53,6 +55,9 @@ class TestCase(BaseTestCase):
 
 
 for name in glob.glob(os.path.join(os.path.dirname(__file__), "test_*.py")):
+    # skip m4a in py3k
+    if sys.version_info[0] != 2 and "test_m4a" in name:
+        continue
     module = "tests." + os.path.basename(name)
     __import__(module[:-3], {}, {}, [])
 
@@ -92,7 +97,7 @@ class Runner(object):
     def run(self, test):
         suite = unittest.makeSuite(test)
         pref = '%s (%d): ' % (test.__name__, len(suite._tests))
-        print pref + " " * (25 - len(pref)),
+        print (pref + " " * (25 - len(pref)), end="")
         result = Result()
         suite(result)
         result.printErrors()
@@ -103,43 +108,46 @@ def unit(run=[], quick=False):
     import mmap
 
     runner = Runner()
-    failures = False
+    failures = 0
+    count = 0
     tests = [t for t in suites if not run or t.__name__ in run]
 
     # normal run, trace mmap calls
     orig_mmap = mmap.mmap
     uses_mmap = []
-    print "Running tests with real mmap."
+    print("Running tests with real mmap.")
     for test in tests:
         def new_mmap(*args, **kwargs):
             if test not in uses_mmap:
                 uses_mmap.append(test)
             return orig_mmap(*args, **kwargs)
         mmap.mmap = new_mmap
-        failures |= runner.run(test)
+        failures += runner.run(test)
     mmap.mmap = orig_mmap
+    count += len(tests)
 
     # make sure the above works
     if not run:
         assert len(uses_mmap) > 1
 
     if quick:
-        return failures
+        return count, failures
 
     # run mmap using tests with mocked lockf
     try:
         import fcntl
     except ImportError:
-        print "Unable to run mocked fcntl.lockf tests."
+        print("Unable to run mocked fcntl.lockf tests.")
     else:
         def MockLockF(*args, **kwargs):
             raise IOError
         lockf = fcntl.lockf
         fcntl.lockf = MockLockF
-        print "Running tests with mocked failing fcntl.lockf."
+        print("Running tests with mocked failing fcntl.lockf.")
         for test in uses_mmap:
-            failures |= runner.run(test)
+            failures += runner.run(test)
         fcntl.lockf = lockf
+        count += len(uses_mmap)
 
     # failing mmap.move
     class MockMMap(object):
@@ -152,18 +160,20 @@ def unit(run=[], quick=False):
         def close(self):
             pass
 
-    print "Running tests with mocked failing mmap.move."
+    print("Running tests with mocked failing mmap.move.")
     mmap.mmap = MockMMap
     for test in uses_mmap:
-        failures |= runner.run(test)
+        failures += runner.run(test)
+    count += len(uses_mmap)
 
     # failing mmap.mmap
     def MockMMap2(*args, **kwargs):
         raise EnvironmentError
 
     mmap.mmap = MockMMap2
-    print "Running tests with mocked failing mmap.mmap."
+    print("Running tests with mocked failing mmap.mmap.")
     for test in uses_mmap:
-        failures |= runner.run(test)
+        failures += runner.run(test)
+    count += len(uses_mmap)
 
-    return failures
+    return count, failures
