@@ -1,9 +1,9 @@
 import os
-from StringIO import StringIO
 from tempfile import mkstemp
 import shutil
 
 from tests import TestCase, add
+from mutagen._compat import cBytesIO, text_type
 from mutagen import File, Metadata, FileType
 from mutagen.oggvorbis import OggVorbis
 from mutagen.oggflac import OggFLAC
@@ -20,6 +20,7 @@ from mutagen.musepack import Musepack
 from mutagen.monkeysaudio import MonkeysAudio
 from mutagen.optimfrog import OptimFROG
 from mutagen.asf import ASF
+from mutagen.aiff import AIFF
 try: from os.path import devnull
 except ImportError: devnull = "/dev/null"
 
@@ -68,7 +69,7 @@ class TFile(TestCase):
     def test_bad(self):
         try: self.failUnless(File(devnull) is None)
         except (OSError, IOError):
-            print "WARNING: Unable to open %s." % devnull
+            print("WARNING: Unable to open %s." % devnull)
         self.failUnless(File(__file__) is None)
 
     def test_empty(self):
@@ -179,9 +180,24 @@ class TFile(TestCase):
         self.failUnless(isinstance(
             File(os.path.join("tests", "data", "silence-3.wma")), ASF))
 
+    def test_aiff(self):
+        data_path = os.path.join("tests", "data")
+        self.failUnless(isinstance(
+            File(os.path.join(data_path, "with-id3.aif")), AIFF))
+        self.failUnless(isinstance(
+            File(os.path.join(data_path, "11k-1ch-2s-silence.aif")), AIFF))
+        self.failUnless(isinstance(
+            File(os.path.join(data_path, "48k-2ch-s16-silence.aif")), AIFF))
+        self.failUnless(isinstance(
+            File(os.path.join(data_path, "8k-1ch-1s-silence.aif")), AIFF))
+        self.failUnless(isinstance(
+            File(os.path.join(data_path, "8k-1ch-3.5s-silence.aif")), AIFF))
+        self.failUnless(isinstance(
+            File(os.path.join(data_path, "8k-4ch-1s-silence.aif")), AIFF))
+
     def test_id3_indicates_mp3_not_tta(self):
-        header = "ID3 the rest of this is garbage"
-        fileobj = StringIO(header)
+        header = b"ID3 the rest of this is garbage"
+        fileobj = cBytesIO(header)
         filename = "not-identifiable.ext"
         self.failUnless(TrueAudio.score(filename, fileobj, header) <
                         MP3.score(filename, fileobj, header))
@@ -189,18 +205,19 @@ class TFile(TestCase):
 add(TFile)
 
 class TFileUpperExt(TestCase):
-    FILES = [(os.path.join("tests", "data", "empty.ofr"), OptimFROG),
-             (os.path.join("tests", "data", "sv5_header.mpc"), Musepack),
-             (os.path.join("tests", "data", "silence-3.wma"), ASF),
-             (os.path.join("tests", "data", "truncated-64bit.mp4"), MP4),
-             (os.path.join("tests", "data", "silence-44-s.flac"), FLAC),
+    FILES = [(os.path.join(b"tests", b"data", b"empty.ofr"), OptimFROG),
+             (os.path.join(b"tests", b"data", b"sv5_header.mpc"), Musepack),
+             (os.path.join(b"tests", b"data", b"silence-3.wma"), ASF),
+             (os.path.join(b"tests", b"data", b"truncated-64bit.mp4"), MP4),
+             (os.path.join(b"tests", b"data", b"silence-44-s.flac"), FLAC),
              ]
              
     def setUp(self):
         checks = []
         for (original, instance) in self.FILES:
-            ext = original.rsplit(".", 1)[-1]
-            fd, filename = mkstemp(suffix='.'+ext.upper())
+            ext = original.rsplit(b".", 1)[-1]
+            suffix = b'.' + ext.upper()
+            fd, filename = mkstemp(suffix=str(suffix))
             os.close(fd)
             shutil.copy(original, filename)
             checks.append((filename, instance))
@@ -208,7 +225,13 @@ class TFileUpperExt(TestCase):
 
     def test_case_insensitive_ext(self):
         for (path, instance) in self.checks:
-            self.failUnless(isinstance(File(path, options=[instance]), instance))
+            if isinstance(path, bytes):
+                path = path.decode("ascii")
+            self.failUnless(
+                isinstance(File(path, options=[instance]), instance))
+            path = path.encode("ascii")
+            self.failUnless(
+                isinstance(File(path, options=[instance]), instance))
 
     def tearDown(self):
         for (path, instance) in self.checks:
