@@ -1,4 +1,5 @@
 from mutagen._util import DictMixin, cdata, utf8, insert_bytes, delete_bytes
+from mutagen._compat import text_type, itervalues, iterkeys, iteritems, PY2
 from tests import TestCase, add
 import random
 
@@ -15,25 +16,25 @@ class FDict(DictMixin):
 class Tutf8(TestCase):
 
     def test_str(self):
-        value = utf8("1234")
-        self.failUnlessEqual(value, "1234")
-        self.failUnless(isinstance(value, str))
+        value = utf8(b"1234")
+        self.failUnlessEqual(value, b"1234")
+        self.failUnless(isinstance(value, bytes))
 
     def test_bad_str(self):
-        value = utf8("\xab\xde")
+        value = utf8(b"\xab\xde")
         # Two '?' symbols.
-        self.failUnlessEqual(value, "\xef\xbf\xbd\xef\xbf\xbd")
-        self.failUnless(isinstance(value, str))
+        self.failUnlessEqual(value, b"\xef\xbf\xbd\xef\xbf\xbd")
+        self.failUnless(isinstance(value, bytes))
 
     def test_low_unicode(self):
         value = utf8(u"1234")
-        self.failUnlessEqual(value, "1234")
-        self.failUnless(isinstance(value, str))
+        self.failUnlessEqual(value, b"1234")
+        self.failUnless(isinstance(value, bytes))
 
     def test_high_unicode(self):
         value = utf8(u"\u1234")
-        self.failUnlessEqual(value, '\xe1\x88\xb4')
-        self.failUnless(isinstance(value, str))
+        self.failUnlessEqual(value, b'\xe1\x88\xb4')
+        self.failUnless(isinstance(value, bytes))
 
     def test_invalid(self):
         self.failUnlessRaises(TypeError, utf8, 1234)
@@ -54,8 +55,9 @@ class TDictMixin(TestCase):
     def test_has_key_contains(self):
         self.failUnless("foo" in self.fdict)
         self.failIf("bar" in self.fdict)
-        self.failUnless(self.fdict.has_key("foo"))
-        self.failIf(self.fdict.has_key("bar"))
+        if PY2:
+            self.failUnless(self.fdict.has_key("foo"))
+            self.failIf(self.fdict.has_key("bar"))
 
     def test_iter(self):
         self.failUnlessEqual(list(iter(self.fdict)), ["foo"])
@@ -68,19 +70,19 @@ class TDictMixin(TestCase):
     def test_keys(self):
         self.failUnlessEqual(list(self.fdict.keys()), list(self.rdict.keys()))
         self.failUnlessEqual(
-            list(self.fdict.iterkeys()), list(self.rdict.iterkeys()))
+            list(iterkeys(self.fdict)), list(iterkeys(self.rdict)))
 
     def test_values(self):
         self.failUnlessEqual(
             list(self.fdict.values()), list(self.rdict.values()))
         self.failUnlessEqual(
-            list(self.fdict.itervalues()), list(self.rdict.itervalues()))
+            list(itervalues(self.fdict)), list(itervalues(self.rdict)))
 
     def test_items(self):
         self.failUnlessEqual(
             list(self.fdict.items()), list(self.rdict.items()))
         self.failUnlessEqual(
-            list(self.fdict.iteritems()), list(self.rdict.iteritems()))
+            list(iteritems(self.fdict)), list(iteritems(self.rdict)))
 
     def test_pop(self):
         self.failUnlessEqual(self.fdict.pop("foo"), self.rdict.pop("foo"))
@@ -135,10 +137,10 @@ add(TDictMixin)
 
 class Tcdata(TestCase):
 
-    ZERO = "\x00\x00\x00\x00"
-    LEONE = "\x01\x00\x00\x00"
-    BEONE = "\x00\x00\x00\x01"
-    NEGONE = "\xff\xff\xff\xff"
+    ZERO = b"\x00\x00\x00\x00"
+    LEONE = b"\x01\x00\x00\x00"
+    BEONE = b"\x00\x00\x00\x01"
+    NEGONE = b"\xff\xff\xff\xff"
 
     def test_int_le(self):
         self.failUnlessEqual(cdata.int_le(self.ZERO), 0)
@@ -163,10 +165,10 @@ class Tcdata(TestCase):
         self.failUnlessEqual(cdata.ulonglong_le(self.NEGONE * 2), 2**64-1)
 
     def test_invalid_lengths(self):
-        self.failUnlessRaises(cdata.error, cdata.int_le, "")
-        self.failUnlessRaises(cdata.error, cdata.longlong_le, "")
-        self.failUnlessRaises(cdata.error, cdata.uint_le, "")
-        self.failUnlessRaises(cdata.error, cdata.ulonglong_le, "")
+        self.failUnlessRaises(cdata.error, cdata.int_le, b"")
+        self.failUnlessRaises(cdata.error, cdata.longlong_le, b"")
+        self.failUnlessRaises(cdata.error, cdata.uint_le, b"")
+        self.failUnlessRaises(cdata.error, cdata.ulonglong_le, b"")
 
     def test_test(self):
         self.failUnless(cdata.test_bit((1), 0))
@@ -199,94 +201,95 @@ class FileHandling(TestCase):
         return fobj.read()
 
     def test_insert_into_empty(self):
-        o = self.file('')
+        o = self.file(b'')
         insert_bytes(o, 8, 0)
-        self.assertEquals('\x00' * 8, self.read(o))
+        self.assertEquals(b'\x00' * 8, self.read(o))
 
     def test_insert_before_one(self):
-        o = self.file('a')
+        o = self.file(b'a')
         insert_bytes(o, 8, 0)
-        self.assertEquals('a' + '\x00' * 7 + 'a', self.read(o))
+        self.assertEquals(b'a' + b'\x00' * 7 + b'a', self.read(o))
 
     def test_insert_after_one(self):
-        o = self.file('a')
+        o = self.file(b'a')
         insert_bytes(o, 8, 1)
-        self.assertEquals('a' + '\x00' * 8, self.read(o))
+        self.assertEquals(b'a' + b'\x00' * 8, self.read(o))
 
     def test_smaller_than_file_middle(self):
-        o = self.file('abcdefghij')
+        o = self.file(b'abcdefghij')
         insert_bytes(o, 4, 4)
-        self.assertEquals('abcdefghefghij', self.read(o))
+        self.assertEquals(b'abcdefghefghij', self.read(o))
 
     def test_smaller_than_file_to_end(self):
-        o = self.file('abcdefghij')
+        o = self.file(b'abcdefghij')
         insert_bytes(o, 4, 6)
-        self.assertEquals('abcdefghijghij', self.read(o))
+        self.assertEquals(b'abcdefghijghij', self.read(o))
 
     def test_smaller_than_file_across_end(self):
-        o = self.file('abcdefghij')
+        o = self.file(b'abcdefghij')
         insert_bytes(o, 4, 8)
-        self.assertEquals('abcdefghij\x00\x00ij', self.read(o))
+        self.assertEquals(b'abcdefghij\x00\x00ij', self.read(o))
 
     def test_smaller_than_file_at_end(self):
-        o = self.file('abcdefghij')
+        o = self.file(b'abcdefghij')
         insert_bytes(o, 3, 10)
-        self.assertEquals('abcdefghij\x00\x00\x00', self.read(o))
+        self.assertEquals(b'abcdefghij\x00\x00\x00', self.read(o))
 
     def test_smaller_than_file_at_beginning(self):
-        o = self.file('abcdefghij')
+        o = self.file(b'abcdefghij')
         insert_bytes(o, 3, 0)
-        self.assertEquals('abcabcdefghij', self.read(o))
+        self.assertEquals(b'abcabcdefghij', self.read(o))
 
     def test_zero(self):
-        o = self.file('abcdefghij')
+        o = self.file(b'abcdefghij')
         self.assertRaises((AssertionError, ValueError), insert_bytes, o, 0, 1)
 
     def test_negative(self):
-        o = self.file('abcdefghij')
+        o = self.file(b'abcdefghij')
         self.assertRaises((AssertionError, ValueError), insert_bytes, o, 8, -1)
 
     def test_delete_one(self):
-        o = self.file('a')
+        o = self.file(b'a')
         delete_bytes(o, 1, 0)
-        self.assertEquals('', self.read(o))
+        self.assertEquals(b'', self.read(o))
 
     def test_delete_first_of_two(self):
-        o = self.file('ab')
+        o = self.file(b'ab')
         delete_bytes(o, 1, 0)
-        self.assertEquals('b', self.read(o))
+        self.assertEquals(b'b', self.read(o))
 
     def test_delete_second_of_two(self):
-        o = self.file('ab')
+        o = self.file(b'ab')
         delete_bytes(o, 1, 1)
-        self.assertEquals('a', self.read(o))
+        self.assertEquals(b'a', self.read(o))
 
     def test_delete_third_of_two(self):
-        o = self.file('ab')
+        o = self.file(b'ab')
         self.assertRaises(AssertionError, delete_bytes, o, 1, 2)
 
     def test_delete_middle(self):
-        o = self.file('abcdefg')
+        o = self.file(b'abcdefg')
         delete_bytes(o, 3, 2)
-        self.assertEquals('abfg', self.read(o))
+        self.assertEquals(b'abfg', self.read(o))
 
     def test_delete_across_end(self):
-        o = self.file('abcdefg')
+        o = self.file(b'abcdefg')
         self.assertRaises(AssertionError, delete_bytes, o, 4, 8)
 
     def test_delete_zero(self):
-        o = self.file('abcdefg')
+        o = self.file(b'abcdefg')
         self.assertRaises(AssertionError, delete_bytes, o, 0, 3)
 
     def test_delete_negative(self):
-        o = self.file('abcdefg')
+        o = self.file(b'abcdefg')
         self.assertRaises(AssertionError, delete_bytes, o, 4, -8)
 
     def test_insert_6106_79_51760(self):
         # This appears to be due to ANSI C limitations in read/write on rb+
         # files. The problematic behavior only showed up in our mmap fallback
         # code for transfers of this or similar sizes. 
-        data = ''.join(map(str, range(12574))) # 51760 bytes
+        data = u''.join(map(text_type, range(12574))) # 51760 bytes
+        data = data.encode("ascii")
         o = self.file(data)
         insert_bytes(o, 6106, 79)
         self.failUnless(data[:6106+79] + data[79:] == self.read(o))
@@ -295,7 +298,8 @@ class FileHandling(TestCase):
         # This appears to be due to ANSI C limitations in read/write on rb+
         # files. The problematic behavior only showed up in our mmap fallback
         # code for transfers of this or similar sizes. 
-        data = ''.join(map(str, range(12574))) # 51760 bytes
+        data = u''.join(map(text_type, range(12574))) # 51760 bytes
+        data = data.encode("ascii")
         o = self.file(data[:6106+79] + data[79:])
         delete_bytes(o, 6106, 79)
         self.failUnless(data == self.read(o))
@@ -316,7 +320,7 @@ class FileHandling(TestCase):
                         min_buffer_size < max_buffer_size,
                         "Given testing parameters make this test useless")
         for j in range(num_runs):
-            data = "ABCDEFGHIJKLMNOPQRSTUVWXYZ" * 1024
+            data = b"ABCDEFGHIJKLMNOPQRSTUVWXYZ" * 1024
             fobj = self.file(data)
             filesize = len(data)
             # Generate the list of changes to apply
