@@ -1,8 +1,8 @@
 import os
 import shutil
 from tempfile import mkstemp
-from cStringIO import StringIO
 
+from mutagen._compat import BytesIO
 from mutagen.oggopus import OggOpus, OggOpusInfo, delete
 from mutagen.ogg import OggPage
 from tests import add
@@ -37,18 +37,18 @@ class TOggOpus(TOggFileType):
     def test_invalid_not_first(self):
         page = OggPage(open(self.filename, "rb"))
         page.first = False
-        self.failUnlessRaises(IOError, OggOpusInfo, StringIO(page.write()))
+        self.failUnlessRaises(IOError, OggOpusInfo, BytesIO(page.write()))
 
     def test_unsupported_version(self):
         page = OggPage(open(self.filename, "rb"))
-        data = list(page.packets[0])
+        data = bytearray(page.packets[0])
 
-        data[8] = "\x03"
-        page.packets[0] = "".join(data)
-        OggOpusInfo(StringIO(page.write()))
+        data[8] = ord(b"\x03")
+        page.packets[0] = bytes(data)
+        OggOpusInfo(BytesIO(page.write()))
 
-        data[8] = "\x10"
-        page.packets[0] = "".join(data)
-        self.failUnlessRaises(IOError, OggOpusInfo, StringIO(page.write()))
+        data[8] = ord(b"\x10")
+        page.packets[0] = bytes(data)
+        self.failUnlessRaises(IOError, OggOpusInfo, BytesIO(page.write()))
 
 add(TOggOpus)
