@@ -1,7 +1,7 @@
 import os
 import shutil
 
-from cStringIO import StringIO
+from mutagen._compat import cBytesIO
 from mutagen.ogg import OggPage
 from mutagen.oggspeex import OggSpeex, OggSpeexInfo, delete
 from tests import add
@@ -35,7 +35,7 @@ class TOggSpeex(TOggFileType):
     def test_invalid_not_first(self):
         page = OggPage(open(self.filename, "rb"))
         page.first = False
-        self.failUnlessRaises(IOError, OggSpeexInfo, StringIO(page.write()))
+        self.failUnlessRaises(IOError, OggSpeexInfo, cBytesIO(page.write()))
 
     def test_vendor(self):
         self.failUnless(
