@@ -1,15 +1,16 @@
 import os
 import sys
-import StringIO
+import imp
+
+from mutagen._compat import StringIO
 
 from tests import TestCase
 
 
 def get_var(tool_name, entry="main"):
     tool_path = os.path.join("tools", tool_name)
-    env = {}
-    execfile(tool_path, env)
-    return env[entry]
+    mod = imp.load_source(tool_name, tool_path)
+    return getattr(mod, entry)
 
 
 class _TTools(TestCase):
@@ -26,11 +27,11 @@ class _TTools(TestCase):
             assert isinstance(arg, str)
         old_stdout = sys.stdout
         try:
-            out = StringIO.StringIO()
+            out = StringIO()
             sys.stdout = out
             try:
                 ret = self._main([self.TOOL_NAME] + list(args))
-            except SystemExit, e:
+            except SystemExit as e:
                 ret = e.code
             ret = ret or 0
             return (ret,  out.getvalue())
